@@ -629,6 +629,12 @@ status_ok (GnomeCD *gcd,
 			/* Update tray icon tooltip */
 			if (gcd->tray_tips)	
 				gtk_tooltips_set_tip (gcd->tray_tips, gcd->tray, _("CD Player"), NULL);
+
+			/* Initialise the display time to 0:00 in stopped state*/
+			cd_display_set_line (CD_DISPLAY (gcd->display),
+					     CD_DISPLAY_LINE_TIME, "0:00");
+			/* Update the position of the slider */
+			gtk_adjustment_set_value (GTK_ADJUSTMENT(gcd->position_adj), 0);
 		}		
 		break;
 		
@@ -922,6 +928,27 @@ playmode_changed_cb (GtkWidget *display,
 	gcd->cdrom->playmode = mode;
 }
 
+/* Popup Menu callback */
+gboolean
+button_press_event_cb (GtkWidget *widget,
+                GdkEventButton *event,
+                GnomeCD *gcd)
+{
+    if ((event->button == 3) && !(event->state & (GDK_SHIFT_MASK | GDK_CONTROL_MASK | GDK_MOD1_MASK))) {
+        make_popup_menu (gcd, event);
+        return TRUE;
+    }
+    return FALSE;
+}
+
+/* Popup Menu callback */
+gboolean
+popup_menu_cb (GtkWidget *widget, GnomeCD *gcd)
+{
+    make_popup_menu (gcd, NULL);
+    return TRUE;
+}
+
 gboolean
 tray_icon_clicked (GtkWidget *widget, GdkEventButton *event, GnomeCD *gcd)
 {
@@ -991,7 +1018,7 @@ open_preferences (GtkWidget *widget,
 	static GtkWidget *dialog = NULL;
 
 	if (dialog == NULL) {
-		dialog = preferences_dialog_show (gcd, FALSE);
+		dialog = preferences_dialog_show (gcd);
 		g_signal_connect (G_OBJECT (dialog), "destroy",
 				  G_CALLBACK (gtk_widget_destroyed), &dialog);
 	} else {
