@@ -120,4 +120,7 @@ GCDTheme *theme_load (GnomeCD    *gcd,
 void theme_change_widgets (GnomeCD *gcd);
 void theme_free (GCDTheme *theme);
 
+void make_popup_menu (GnomeCD *gcd,
+                GdkEventButton *event);
+
 #endif
